//
// CActiveConnect
//

// A CActive derived Connection class for Internet sockets

#include "ActiveConnect.h"
#include "ACObservers.h"

#include <nifman.h>

#define CACTIVECONNECT_CATEGORY _L("CActiveConnect")

CActiveConnect::CActiveConnect(RSocket& aSocket) 
: CActive(CActive::EPriorityStandard)
, iSocket(aSocket)
{
}


CActiveConnect::~CActiveConnect() {

    Cancel();

    iAddrDB.Close();
    iCommServer.Close();
}

void CActiveConnect::ConstructL( const TProtocolName& aName
			       , MConnectionObserver* aObserver
			       ) {
    iObserver = aObserver;
    User::LeaveIfNull(iObserver);

    iPhase          = EDisconnected;
    iDisconnectType = EDisconnectSocket;

    User::LeaveIfError(iCommServer.Connect());
    User::LeaveIfError(iCommServer.FindProtocol(aName, iProtocol));

    User::LeaveIfError(iAddrDB.Open(iCommServer, iProtocol.iAddrFamily, iProtocol.iProtocol));

    CActiveScheduler::Add(this); 
}


CActiveConnect* CActiveConnect::NewL( MConnectionObserver* aObserver
				    , const TProtocolName& aName
				    , RSocket&             aSocket
				    ) {
    
    CActiveConnect* self = CActiveConnect::NewLC(aObserver, aName, aSocket);
    CleanupStack::Pop();
    return self;
}

CActiveConnect* CActiveConnect::NewLC( MConnectionObserver* aObserver
				     , const TProtocolName& aName
				     , RSocket&             aSocket
				     ) {
    
    CActiveConnect* self = new (ELeave) CActiveConnect(aSocket);
    CleanupStack::PushL(self);
    self->ConstructL(aName, aObserver);
    return self;
}


void CActiveConnect::StartProtocol() {

    if (iPhase != EDisconnected) {
	User::Panic(CACTIVECONNECT_CATEGORY, EWrongPhase);
    }

    iCommServer.StartProtocol( iProtocol.iAddrFamily
	                     , iProtocol.iSockType
			     , iProtocol.iProtocol
			     , iStatus
			     );

    iPhase = EStartingServer;
    SetActive();
}


void CActiveConnect::ResolveAddress() {

    if (iPhase != EStartingServer) {
	User::Panic(CACTIVECONNECT_CATEGORY, EWrongPhase);
    }

    iObserver->OnServerStart(iStatus);
    if (iStatus == KErrNone) {

	iAddrDB.GetByName(iPeerName, iResolvedAddr, iStatus);

	iPhase = EResolvingAddress;
	SetActive();
    }
    else {
	iPhase = EDisconnected;
    }
}


void CActiveConnect::CompleteConnectL() {

    if (iPhase != EResolvingAddress) {
	User::Panic(CACTIVECONNECT_CATEGORY, EWrongPhase);
    }

    iObserver->OnAddressResolved(iStatus);

    if (iStatus == KErrNone) {

	iResolvedAddr().iAddr.SetPort(iPeerPort);

	User::LeaveIfError(iSocket.Open( iCommServer
	                               , iProtocol.iAddrFamily
				       , iProtocol.iSockType
				       , iProtocol.iProtocol
				       )
		          );
	
	iSocket.Connect(iResolvedAddr().iAddr, iStatus);

	iPhase = EConnecting;
	SetActive();
    }
    else {
	iPhase = EDisconnected;
    }
}

void CActiveConnect::OnSocketConnect() {
    if (iPhase != EConnecting) {
	User::Panic(CACTIVECONNECT_CATEGORY, EWrongPhase);
    }
    
    iObserver->OnConnect(iStatus);

    if (iStatus == KErrNone) {
	iPhase = EConnected;
    }
    else {
	iPhase = EDisconnected;
    }
}


void CActiveConnect::CloseConnection() {

    if (iPhase != EConnected) {
	User::Panic(CACTIVECONNECT_CATEGORY, EWrongPhase);
    }
    
    iSocket.CancelAll();
    iSocket.Shutdown(RSocket::ENormal, iStatus);

    iPhase = EDisconnecting;
    SetActive();
}



void CActiveConnect::Connect( const THostName&     aPeerName
			    , TUint                aPort
			    ) {

    iPeerName.Copy(aPeerName);
    iPeerPort = aPort;

    StartProtocol();
}


void CActiveConnect::Disconnect(TDisconnectType t) {
    iDisconnectType = t;

    switch (iPhase) {
    case EStartingServer:
    case EResolvingAddress:
    case EConnecting:
	Cancel();
	if (iDisconnectType == EDisconnectSocketAndNet) {
	    DisconnectNetwork();
	}
	break;
    
    case EConnected:
	CloseConnection();
	break;

    case EDisconnecting:
    case EDisconnected:
	if (iDisconnectType == EDisconnectSocketAndNet) {
	    DisconnectNetwork();
	}
	break;
    }
}



void CActiveConnect::RunL() {

    switch (iPhase) {
    case EStartingServer:
	ResolveAddress();
	break;

    case EResolvingAddress:
	CompleteConnectL();
	break;

    case EConnecting:
	OnSocketConnect();
	break;
    
    case EDisconnecting:
	iSocket.Close();
	iObserver->OnDisconnect(iStatus);
	if (iDisconnectType == EDisconnectSocketAndNet) {
	    DisconnectNetwork();
	}
	iPhase = EDisconnected;
	break;
    }
}
    
// Cancel request.
// Defined as pure virtual by CActive;
// implementation provided by this class.
void CActiveConnect::DoCancel() {

    switch (iPhase) {
    case EStartingServer:   // nothing to do!
	break;

    case EResolvingAddress:
	iAddrDB.Cancel();
	break;

    case EConnecting:
	iSocket.CancelConnect();
	break;
    }

    if (iPhase != EDisconnected) {
	iObserver->OnDisconnect(KErrCancel);
	iPhase = EDisconnected;
    }
}


//
// DisconnectNetwork
//
// Disconnect any Dial-out connection.
void CActiveConnect::DisconnectNetwork() {
    RNif nif;
    if (nif.Open() == KErrNone) {
	nif.Stop();
	nif.Close();
    }

    // reset so we never disconnect the net unintentionally
    iDisconnectType = EDisconnectSocket;
}

